package javax.swing.plaf.synth;

import java.awt.event.FocusEvent;
import java.awt.event.ContainerEvent;
import java.awt.event.FocusListener;
import java.awt.event.ContainerListener;
import java.awt.Insets;
import javax.swing.border.AbstractBorder;
import java.beans.PropertyChangeEvent;
import javax.swing.JViewport;
import java.awt.Component;
import javax.swing.text.JTextComponent;
import javax.swing.UIManager;
import javax.swing.plaf.UIResource;
import javax.swing.JScrollPane;
import java.awt.Rectangle;
import javax.swing.border.Border;
import java.awt.Graphics;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicScrollPaneUI;

public class SynthScrollPaneUI extends BasicScrollPaneUI implements PropertyChangeListener, SynthUI
{
    private SynthStyle style;
    private boolean viewportViewHasFocus;
    private SynthScrollPaneUI.ViewportViewFocusHandler viewportViewFocusHandler;
    
    public SynthScrollPaneUI() {
        this.viewportViewHasFocus = false;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthScrollPaneUI();
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintScrollPaneBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        Border viewportBorder = this.scrollpane.getViewportBorder();
        if (viewportBorder != null) {
            Rectangle viewportBorderBounds = this.scrollpane.getViewportBorderBounds();
            viewportBorder.paintBorder(this.scrollpane, graphics, viewportBorderBounds.x, viewportBorderBounds.y, viewportBorderBounds.width, viewportBorderBounds.height);
        }
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintScrollPaneBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    protected void installDefaults(JScrollPane scrollPane) {
        this.updateStyle(scrollPane);
    }
    
    private void updateStyle(JScrollPane scrollPane) {
        SynthContext context = this.getContext(scrollPane, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style) {
            Border viewportBorder = this.scrollpane.getViewportBorder();
            if (viewportBorder == null || viewportBorder instanceof UIResource) {
                this.scrollpane.setViewportBorder(new SynthScrollPaneUI.ViewportBorder(context));
            }
            if (style != null) {
                this.uninstallKeyboardActions(scrollPane);
                this.installKeyboardActions(scrollPane);
            }
        }
        context.dispose();
    }
    
    @Override
    protected void installListeners(JScrollPane c) {
        super.installListeners(c);
        c.addPropertyChangeListener(this);
        if (UIManager.getBoolean("ScrollPane.useChildTextComponentFocus")) {
            this.viewportViewFocusHandler = new SynthScrollPaneUI.ViewportViewFocusHandler();
            c.getViewport().addContainerListener(this.viewportViewFocusHandler);
            Component view = c.getViewport().getView();
            if ((view instanceof JTextComponent)) {
                view.addFocusListener(this.viewportViewFocusHandler);
            }
        }
    }
    
    @Override
    protected void uninstallDefaults(JScrollPane scrollPane) {
        SynthContext context = this.getContext(scrollPane, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        if ((this.scrollpane.getViewportBorder() instanceof UIResource)) {
            this.scrollpane.setViewportBorder(null);
        }
    }
    
    @Override
    protected void uninstallListeners(JComponent c) {
        super.uninstallListeners(c);
        c.removePropertyChangeListener(this);
        if (this.viewportViewFocusHandler != null) {
            JViewport viewport = ((JScrollPane)c).getViewport();
            viewport.removeContainerListener(this.viewportViewFocusHandler);
            if (viewport.getView() != null) {
                viewport.getView().removeFocusListener(this.viewportViewFocusHandler);
            }
            this.viewportViewFocusHandler = null;
        }
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, this.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    private int getComponentState(JComponent component) {
        int componentState = SynthLookAndFeel.getComponentState(component);
        if (this.viewportViewFocusHandler != null && this.viewportViewHasFocus) {
            componentState |= 0x100;
        }
        return componentState;
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle(this.scrollpane);
        }
    }
    
    private class ViewportBorder extends AbstractBorder implements UIResource
    {
        private Insets insets;
        
        ViewportBorder(SynthContext synthContext) {
            this.insets = (Insets)synthContext.getStyle().get(synthContext, "ScrollPane.viewportBorderInsets");
            if (this.insets == null) {
                this.insets = SynthLookAndFeel.EMPTY_UIRESOURCE_INSETS;
            }
        }
        
        @Override
        public void paintBorder(Component component, Graphics graphics, int n, int n2, int n3, int n4) {
            SynthContext context = SynthScrollPaneUI.this.getContext((JComponent)component);
            if (context.getStyle() == null) {
                assert false : "SynthBorder is being used outside after the  UI has been uninstalled";
                return;
            }
            else {
                context.getPainter().paintViewportBorder(context, graphics, n, n2, n3, n4);
                context.dispose();
                return;
            }
        }
        
        @Override
        public Insets getBorderInsets(Component component, Insets insets) {
            if (insets == null) {
                return new Insets(this.insets.top, this.insets.left, this.insets.bottom, this.insets.right);
            }
            insets.top = this.insets.top;
            insets.bottom = this.insets.bottom;
            insets.left = this.insets.left;
            insets.right = this.insets.left;
            return insets;
        }
        
        @Override
        public boolean isBorderOpaque() {
            return false;
        }
    }
    
    private class ViewportViewFocusHandler implements ContainerListener, FocusListener
    {
        @Override
        public void componentAdded(ContainerEvent containerEvent) {
            if ((containerEvent.getChild() instanceof JTextComponent)) {
                containerEvent.getChild().addFocusListener(this);
                SynthScrollPaneUI.this.viewportViewHasFocus = containerEvent.getChild().isFocusOwner();
                SynthScrollPaneUI.this.scrollpane.repaint();
            }
        }
        
        @Override
        public void componentRemoved(ContainerEvent containerEvent) {
            if ((containerEvent.getChild() instanceof JTextComponent)) {
                containerEvent.getChild().removeFocusListener(this);
            }
        }
        
        @Override
        public void focusGained(FocusEvent focusEvent) {
            SynthScrollPaneUI.this.viewportViewHasFocus = true;
            SynthScrollPaneUI.this.scrollpane.repaint();
        }
        
        @Override
        public void focusLost(FocusEvent focusEvent) {
            SynthScrollPaneUI.this.viewportViewHasFocus = false;
            SynthScrollPaneUI.this.scrollpane.repaint();
        }
    }
}
